#!/usr/bin/python

import pygame
from pygame.locals import *
from random import *
from vec2d import vec2d


# define custom events
SPAWNBUNNY = USEREVENT + 1
KILLBUNNY = USEREVENT + 2
CONTAGION = USEREVENT + 3
INFECTION = USEREVENT + 4
EXPLOSION = USEREVENT + 5
PLAYERKILLBUNNY = USEREVENT + 6

# define custom event behaviors

class Bunny(pygame.sprite.Sprite):
    '''base class for bunnies'''    
    def __init__(self, screen, init_position, init_direction, speed=.1, img_filename='bluebunny.png'):
#        self.size = (20, 20) # new Bunny objects are 20x20 by default
#        self.color = (255, 255, 255) # new Bunny objects are white by default
        pygame.sprite.Sprite.__init__(self)
        self.screen = screen
        self.speed = speed
        self.image = pygame.image.load(img_filename) #.convert_alpha()
        self.image = self.image.convert_alpha()
        self.image = pygame.transform.smoothscale(self.image, (66,90))
        self.pos = vec2d(init_position)
        self.direction = vec2d(init_direction).normalized()
        self.rect = pygame.Rect(self.pos, self.image.get_size())

        self.time = randint(2000, 8000) # 1000
        self.Action = pygame.event.Event(SPAWNBUNNY, {'parent': '{0} - {1}'.format(self.__class__, self.time)})
        #pygame.time.set_timer(SPAWNBUNNY, self.time)

        self.image_w, self.image_h = self.image.get_size()

        self.hitbox = pygame.Rect((self.pos.x - self.image_w /2, self.pos.y - self.image_h / 2), self.image.get_size())
        
        self.moving = True 
    
    def __str__(self):
        return '{0} - {1}'.format(self.__class__, self.time) 
    
    def update(self, time_passed, bunnies):

        self._Action(time_passed) 
        self._change_direction(time_passed)
        self._move(time_passed, self.moving)

    def blitme(self):
        draw_pos = self.image.get_rect().move(self.pos.x - self.image_w / 2, self.pos.y - self.image_h / 2)
        self.screen.blit(self.image, draw_pos)
    
    _move_counter = 0

    def _move(self, time_passed, moving):
        self._move_counter += time_passed
        if moving:
            displacement = vec2d(self.direction.x * self.speed * time_passed, self.direction.y * self.speed * time_passed)
            self.pos += displacement
            bounds_rect = self.screen.get_rect().inflate(-self.image_w, -self.image_h)
            
            if self.pos.x < bounds_rect.left:
                self.pos.x = bounds_rect.left
                self.direction.x *= -1
            elif self.pos.x > bounds_rect.right:
                self.pos.x = bounds_rect.right
                self.direction.x *= -1
            elif self.pos.y < bounds_rect.top:
                self.pos.y = bounds_rect.top
                self.direction.y *= -1
            elif self.pos.y > bounds_rect.bottom:
                self.pos.y = bounds_rect.bottom
                self.direction.y *= -1

            self.hitbox = pygame.Rect((self.pos.x - self.image_w /2, self.pos.y - self.image_h / 2), self.image.get_size())
        
        move_threshold = choice([i for i in range(800, 3050, 50)])
        if self._move_counter > move_threshold:
            self.moving = not moving
            self._move_counter = 0

    _dir_counter = 0

    def _change_direction(self, time_passed):
        self._dir_counter += time_passed
        if self._dir_counter > randint(400, 800):
            self.direction.rotate(randint(-45, 45))
            self._dir_counter = 0

    _act_counter = 0

    def _Action(self, time_passed):
        self._act_counter += time_passed
        if self._act_counter > self.time:
            if isinstance(self, HealthyBunny):
                pygame.event.post(self.Action)
                self._act_counter = 0
            else: pass

class ExplosionGraphic(pygame.Surface):
    '''explosion graphics'''
    def __init__(self, screen, bomber, blast_radius):
        self.bomber = bomber
        self.screen = screen
        if isinstance(self.bomber, BombBunny):
            self.img_filename = 'kaboom.png'
            sound = pygame.mixer.Sound('explosion.wav')
            sound.play()
        else:
            self.img_filename = 'gas_cloud.png'
            sound = pygame.mixer.Sound('gas_cloud.wav')
            sound.play()
        self.boom = pygame.image.load(self.img_filename)
        #self.boom = self.boom.set_alpha(150)
        self.boom = pygame.transform.smoothscale(self.boom,(int(bomber.image_w * 2), int((bomber.image_h * 2))))
        self.pos = (bomber.pos.x - bomber.image_w / 2, bomber.pos.y - bomber.image_h /2 )
        
    def update(self, time_passed):
        
        return self._fade(time_passed)

    def blitme(self):
#        draw_pos = self.boom.get_rect().move(self.bomber.pos.x - self.boom.image_w / 2, self.bomber.pos.y - self.boom.image_h / 2)
        self.screen.blit(self.boom, self.pos)

    _lifetime = 0
    def _fade(self, time_passed):
        self._lifetime += time_passed
        if self._lifetime > 2000:
            return True
        else: 
            return False

class MutantBunny(Bunny):
    '''base class for mutant bunnies'''
    def __init__(self, screen, init_pos, init_direction, speed):
        Bunny.__init__(self, screen, init_pos, init_direction, speed)
        self.time = randint(2000, 8000)
        self.killer = False
    
    def update(self, time_passed, bunnies):

        self._change_direction(time_passed)
        self._Action(time_passed)
        self._is_Killer(time_passed)
        self._move(time_passed, self.moving)

        if self.killer:
            for bunny in [live_bunny for live_bunny in bunnies if isinstance(live_bunny, HealthyBunny)]:
                if self.hitbox.colliderect(bunny.hitbox):
                    if isinstance(self, KillerBunny):
                        event_type = KILLBUNNY
                    elif isinstance(self, InfectiousBunny):
                        event_type = INFECTION
                    else:
                        continue
                    bunny_action = pygame.event.Event(event_type, {'target': bunny})
                    pygame.event.post(bunny_action)
                else:
                    continue

    _kill_counter = 0
    _kill_time = 3000

    def _is_Killer(self, time_passed):
        self._kill_counter += time_passed
        if self._kill_counter > self._kill_time:
            self.killer = True

class ExplodingBunny(Bunny):
    '''base class for bunnies that explode'''
    def __init__(self, screen, init_pos, init_direction, speed):
        Bunny.__init__(self, screen, init_pos, init_direction, speed)
        self.exploding = False
        self.KABOOM = False
        self.time_to_explode = 5000
        self.fuse_length = 3000

    def update(self, time_passed, bunnies):

        if not self.exploding:        
            self._change_direction(time_passed)
            self._move(time_passed, self.moving)
            self._wait_to_explode(time_passed)
        elif not self.KABOOM:
            self._move(0, False)
            self._wait_to_explode(time_passed)
        else:
            self._explode(bunnies)

    def blitme(self):
        draw_pos = self.image.get_rect().move(self.pos.x - self.image_w / 2, self.pos.y - self.image_h / 2)
        self.screen.blit(self.image, draw_pos)
   
    _fuse = 0
    _blast_counter = 0
    _post_blast = 0


    def _wait_to_explode(self, time_passed):
        self._blast_counter += time_passed
        if self.exploding:
            self._fuse += time_passed
            if self._fuse > self.fuse_length:
                self.KABOOM = True
        else:    
            if self._blast_counter > self.time_to_explode:
                self.exploding = True

    def _explode(self, bunnies):
        blast_radius = pygame.rect.Rect((self.pos.x - (self.image_w * 2.5) / 2, self.pos.y - (self.image_h * 2.5) / 2), (self.image_w * 2.5, self.image_h * 2.5))
        d = {'bomber': self, 'victims': [], 'blast_zone': blast_radius}
        for bunny in bunnies:
            if blast_radius.colliderect(bunny.hitbox):
                d['victims'].append(bunny)
        if d['victims']:
            bunny_action = pygame.event.Event(EXPLOSION, d)
            pygame.event.post(bunny_action)
        else:
            pass

    _move_counter = 0

    def _move(self, time_passed, moving):
        self._move_counter += time_passed
        if moving:
            displacement = vec2d(self.direction.x * self.speed * time_passed, self.direction.y * self.speed * time_passed)
            self.pos += displacement
            bounds_rect = self.screen.get_rect().inflate(-self.image_w, -self.image_h)
            
            if self.pos.x < bounds_rect.left:
                self.pos.x = bounds_rect.left
                self.direction.x *= -1
            elif self.pos.x > bounds_rect.right:
                self.pos.x = bounds_rect.right
                self.direction.x *= -1
            elif self.pos.y < bounds_rect.top:
                self.pos.y = bounds_rect.top
                self.direction.y *= -1
            elif self.pos.y > bounds_rect.bottom:
                self.pos.y = bounds_rect.bottom
                self.direction.y *= -1

            self.hitbox = pygame.Rect((self.pos.x - self.image_w /2, self.pos.y - self.image_h / 2), self.image.get_size())
        
        move_threshold = choice([i for i in range(800, 3050, 50)])
        if self._move_counter > move_threshold:
            self.moving = not moving
            self._move_counter = 0

    _dir_counter = 0

    def _change_direction(self, time_passed):
        self._dir_counter += time_passed
        if self._dir_counter > randint(400, 800):
            self.direction.rotate(randint(-45, 45))
            self._dir_counter = 0

    _act_counter = 0

    def _Action(self, time_passed):
        self._act_counter += time_passed
        if self._act_counter > self.time:
            if isinstance(self, HealthyBunny):
                pygame.event.post(self.Action)
                self._act_counter = 0
            else: pass


class HealthyBunny(Bunny):
    '''a bunny which is not yet a mutant.'''
    def __init__(self, screen, init_pos, init_direction, speed):
        Bunny.__init__(self, screen, init_pos, init_direction, speed)
        self.time = randint(6000, 11000)

        self.point_val = -5

class KillerBunny(MutantBunny):
    '''a mutant bunny with a thirst for bunny blood'''
    def __init__(self, screen, init_pos, init_direction, speed):
        MutantBunny.__init__(self, screen, init_pos, init_direction, speed)
        self.image = pygame.image.load('killerbunny2.png') #.convert_alpha()
        self.image = self.image.convert_alpha()
        self.image = pygame.transform.smoothscale(self.image,(90,82))
        self.time = randint(2000, 8000)

        self.point_val = 10

        def _Action(self):
            pass

class ContagiousBunny(ExplodingBunny):
    '''a mutant bunny that converts HealthyBunnies into more ContagiousBunnies'''
    def __init__(self, screen, init_pos, init_direction, speed):
        ExplodingBunny.__init__(self, screen, init_pos, init_direction, speed)
        self.image = pygame.image.load('contagious_bunny.png')
        self.image = pygame.transform.smoothscale(self.image,(114, 134))
        self.time = 5000

        self.point_val = 30

class InfectiousBunny(MutantBunny):
    '''a mutant bunny that causes mutations in HealthyBunnies'''
    def __init__(self, screen, init_pos, init_direction, speed):
        MutantBunny.__init__(self, screen, init_pos, init_direction, speed)
        self.time = randint(2000, 8000)
        self.image = pygame.image.load('punkbunny.png') #.convert_alpha()
        self.image = self.image.convert_alpha()
        self.image = pygame.transform.smoothscale(self.image,(41,73))

        def _Action(self):
            pass

        self.point_val = 15

class BombBunny(ExplodingBunny):
    '''BombBunny EXPLODES ALL OVER THE PLACE!!!'''
    def __init__(self, screen, init_pos, init_direction, speed):
        ExplodingBunny.__init__(self, screen, init_pos, init_direction, speed)
        self.time = 5000
        self.image = pygame.image.load('bunnybomb.png') #.convert_alpha()
        self.image = self.image.convert_alpha()
        self.image = pygame.transform.smoothscale(self.image,(66,60))

        self.point_val = 20
