#!/usr/bin/python

import time
import pygame
import sys
from pygame.locals import *
from random import *
from BunnyDefinitions import *
import Screens

def main(replay):
    
    SCREEN_WIDTH, SCREEN_HEIGHT = 900, 600
    NUM_BUNNIES = 5
    
    #"Colors----"
    black = (0,0,0)
    white = (255,255,255)
    green = (0,255,0)
    red = (255,0,0)
    dark_red = (139,0,0)
    purple = (205,0,205)
    gold = (255,128,0)
    #"/End Colors----"
    
    pygame.init()
    display = pygame.display
    screen = display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
    display.set_caption('Hare Trigger')
    background = pygame.image.load("grass.jpg")
    background = pygame.transform.smoothscale(background, (SCREEN_WIDTH, SCREEN_HEIGHT))
    
    #Describe font types ----  I should have named these according to their size.  Fail.
    go_font = pygame.font.Font(None, 180)
    title_font = pygame.font.Font(None, 110)
    mid_title_font = pygame.font.Font(None, 80)
    scoreboard_font = pygame.font.Font(None, 65)
    sub_title_font = pygame.font.Font(None, 30)
    #End fonts ----
    
    scoreboard = scoreboard_font.render("Score:", True, gold) #render scoreboard

    clock = pygame.time.Clock()

    current_score = 0
    
    if replay == 0: #code to show intro screen unless they're playing again
        Screens.display('intro',current_score,current_score,current_score)

    def BUNNY_DEFAULTS(): 
        pos = (randint(0, SCREEN_WIDTH), randint(0, SCREEN_HEIGHT))
        dir = (choice([-1, 1]), choice([-1, 1])) 
        speed = .2

        return (pos, dir, speed)

    def new_bunny(type='HealthyBunny'):
        if type == 'HealthyBunny':
            new = HealthyBunny(screen, *BUNNY_DEFAULTS())
        elif type == 'KillerBunny':
            new = KillerBunny(screen, *BUNNY_DEFAULTS())
        elif type == 'ContagiousBunny':
            new = ContagiousBunny(screen, *BUNNY_DEFAULTS())
        elif type == 'InfectiousBunny':
            new = InfectiousBunny(screen, *BUNNY_DEFAULTS())
        elif type == 'BombBunny':
            new = BombBunny(screen, *BUNNY_DEFAULTS())

        return new

    def spawn_bunny():
        x = randrange(100)
        if x in range(97, 100):
            type = 'ContagiousBunny'
            sound = pygame.mixer.Sound('contagious_bunny.wav')
            sound.play()
        elif x in range(90, 97):
            type = 'BombBunny'
        elif x in range(82, 90):
            type = 'InfectiousBunny'
        elif x in range(65, 82):
            type = 'KillerBunny'
        else:
            type = 'HealthyBunny'

        return new_bunny(type)

    def game_over(score, timer, seconds):
        global time_played, converted_time, start_time

        if Screens.display('gameover', score, timer,seconds) == True: # true means the quit the game
            sys.exit()
        elif Screens.display('gameover', score,timer,seconds) == False:
            main(1)

    def bunny_kills_bunny(target):
        sound = pygame.mixer.Sound('killerbunny2.wav')
        sound.play()
        target.kill()
        del target

    def bunny_explosion(bomber, victims, blast_radius):
        for victim in victims:
            victim.kill()
            del victim
            if isinstance(bomber, ContagiousBunny):
                bunnies.add(new_bunny('ContagiousBunny'))
        bomber.kill()
        del bomber

    
    def player_kills_bunny(bunny, score):
        sound = pygame.mixer.Sound('SPLAT.wav')
        sound.play()
        if isinstance(bunny, HealthyBunny):
            bunny.kill()
            del bunny
            score = score - 5
        else:
            bunny.kill()
            score = score + bunny.point_val
            del bunny
        return score

    explosions = []

    bunnies = pygame.sprite.Group()
    for i in range(NUM_BUNNIES):
        bunnies.add(new_bunny('HealthyBunny'))

    current_timer = 0
   
    max_healthy_bunnies = 5

    while 1:
        score = scoreboard_font.render(str(current_score), True, gold) #and point value
        time_passed = clock.tick(30)

        current_timer += time_passed
        current_seconds = int(round(current_timer / 1000, 0))
        minsec = [str(i) for i in divmod(current_seconds, 60)]
        if len(minsec[1]) == 1:
            minsec[1] = '0' + minsec[1]
        time_display = "{0}:{1}".format(*minsec)

        screen.blit(background,[0,0]) #draw background
        score = scoreboard_font.render(str(current_score), True, gold) #prerender point value
        timer = mid_title_font.render(time_display, True, gold)
        
        eventlist = pygame.event.get()
        for game_event in eventlist:
            if game_event.type == pygame.QUIT:
                sys.exit()
            elif game_event.type == pygame.MOUSEBUTTONDOWN:
                for bunny in bunnies:
                    if bunny.hitbox.collidepoint(game_event.pos):
                        current_score = player_kills_bunny(bunny, current_score)
                    continue
            elif game_event.type == SPAWNBUNNY:
                bunnies.add(spawn_bunny())
                continue
            elif game_event.type == KILLBUNNY:
                bunny_kills_bunny(game_event.target)
                continue
            elif game_event.type == INFECTION:
                bunny_kills_bunny(game_event.target)
                bunnies.add(new_bunny('InfectiousBunny'))
                continue
            elif game_event.type == EXPLOSION:
                bunny_explosion(game_event.bomber, game_event.victims, game_event.blast_zone)
                explosions.append(ExplosionGraphic(screen, game_event.bomber, game_event.blast_zone))
        if len([remaining for remaining in bunnies if isinstance(remaining, HealthyBunny)]): 
            num_healthy_bunnies = len([remaining for remaining in bunnies if isinstance(remaining, HealthyBunny)])
            if num_healthy_bunnies > max_healthy_bunnies:
                max_healthy_bunnies = num_healthy_bunnies
        else:
            current_score = current_score + 10*max_healthy_bunnies
            game_over(current_score, timer, current_seconds) 
            

        for bunny in [live_bunny for live_bunny in bunnies]:
            bunny.update(time_passed, bunnies)
            bunny.blitme()

        for explosion in explosions:
            if explosion.update(time_passed):
                del explosion
            else:
                explosion.blitme()

        #--common display stuff
        screen.blit(scoreboard,[340,7])
        screen.blit(score,[480,9])
        screen.blit(timer,[780,9])
        pygame.display.flip()

if __name__ == '__main__':
    main(0)
